	* Definitions for
	* Theo Markettos' Home Automation System
	* 
	* This code is copyright Theo Markettos 1997,
	* and may be freely distributed, with the proviso
	* that it is used for educational and non-profit
	* making purposes only
	* 
	* If you have any problems using this code, you
	* should contact the author at
	* theomarkettos@letterbox.com

* Register sets

ALLREGS	REG	D0-D7/A0-A6		* Register set for all registers
DATAREGS	REG	D0-D7		* Data register set
ADRREGS	REG	A0-A6			* GP Address registers
ALLADRREGS	REG	A0-A7		* All Address registers

* Hardware addresses

IO_BASE	EQU	$80000			* Allow shifting of IO space

ACIA	EQU	IO_BASE+$000
ACIA_SR	EQU	IO_BASE+$000		* Status register  - read only
ACIA_CR	EQU	IO_BASE+$000		* Control register - write only
ACIA_RR	EQU	IO_BASE+$002		* Receive register - read only
ACIA_TR	EQU	IO_BASE+$002		* Transmit register- write only
SER_BPS	EQU	IO_BASE+$004		* Serial BPS and control lines

PIA	EQU	IO_BASE+$200
PIA_PRA	EQU	IO_BASE+$200		* Peripheral register A (CRA bit 2 set)
PIA_DDRA	EQU	IO_BASE+$200	* Data direction register A (CRA bit 2 clear)
PIA_CRA	EQU	IO_BASE+$202		* Control register A
PIA_PRB	EQU	IO_BASE+$204		* Peripheral register B (CRB bit 2 set)
PIA_DDRB	EQU	IO_BASE+$204	* Data direction register B (CRB bit 2 clear)
PIA_CRB	EQU	IO_BASE+$206		* Control register B

VIA	EQU	IO_BASE+$300
VIA_DRB	EQU	IO_BASE+$300		* Data register B
VIA_DRA	EQU	IO_BASE+$302		* Data register A
VIA_DDRB	EQU	IO_BASE+$304	* Data direction register B
VIA_DDRA	EQU	IO_BASE+$306	* Data direction register A
VIA_T1CL	EQU	IO_BASE+$308	* T1 Low counter
VIA_T1CH	EQU	IO_BASE+$30A	* T1 High counter
VIA_T1LL	EQU	IO_BASE+$30C	* T1 Low latches
VIA_T1LH	EQU	IO_BASE+$30E	* T1 High latches
VIA_T2CL	EQU	IO_BASE+$310	* T2 Low counter
VIA_T2CH	EQU	IO_BASE+$312	* T2 High counter
VIA_SR		EQU	IO_BASE+$314	* Shift Register
VIA_ACR		EQU	IO_BASE+$316	* Auxiliary Control Register
VIA_PCR		EQU	IO_BASE+$318	* Peripheral Control Register
VIA_IFR		EQU	IO_BASE+$31A	* Interrupt Flag Register
VIA_IER		EQU	IO_BASE+$31C	* Interrupt Enable Register
VIA_NHDRA	EQU	IO_BASE+$31E	* No Handshake DRA

DIN	EQU	IO_BASE+$600		* Digital In
DOUT	EQU	IO_BASE+$600		* Digital Out
DOUT_H	EQU	IO_BASE+$600		* Digital Out - high byte
DOUT_L	EQU	IO_BASE+$601		* Digital Out - low byte

RTC_SECS	EQU	$00		* Real Time Clock internal addresses
RTC_SECS_ALRM	EQU	$01
RTC_MINS	EQU	$02
RTC_MINS_ALRM	EQU	$03
RTC_HRS		EQU	$04
RTC_HRS_ALRM	EQU	$05
RTC_DAY		EQU	$06
RTC_DATE	EQU	$07
RTC_MONTH	EQU	$08
RTC_YEAR	EQU	$09
RTC_STATUS_A	EQU	$0A
RTC_STATUS_B	EQU	$0B
RTC_STATUS_C	EQU	$0C
RTC_STATUS_D	EQU	$0D
RTC_DATE_ALRM	EQU	$0E
RTC_MONTH_ALRM	EQU	$0F
RTC_YEAR_ALRM	EQU	$10

RTC_DEVICE_ALRM	EQU	$0E
RTC_DATA_VALUE	EQU	$0F

* Software interrupt numbers

OS_Reset	EQU	$0		* SWI number
OS_Stop		EQU	$1

Serial_GetByte	EQU	$0
Serial_PutByte	EQU	$1
Hardware_RTC_Write	EQU	$900
Hardware_RTC_Read	EQU	$901

* RAM allocations

RAM_BASE	EQU	$40000		* Allow shifting of RAM space

SERIAL_BUFLEN	EQU	$FF		* Size of receive and transmit buffers
SERIAL_STATUS	EQU	RAM_BASE+$000	* Store serial status on IRQ
SERIAL_RX_INPTR	EQU	RAM_BASE+$002 * Pointer to store incoming data at
SERIAL_RX_OUTPTR	EQU	RAM_BASE+$004 * Pointer to read from
SERIAL_TX_INPTR	EQU	RAM_BASE+$006 * Pointer to write to
SERIAL_TX_OUTPTR	EQU	RAM_BASE+$008 * Pointer to outputting data
SERIAL_RX_OVERFLOW	EQU	RAM_BASE+$00A	* Flag to show if overflows
SERIAL_TX_OVERFLOW	EQU	RAM_BASE+$00C	* Flag to show if overflows
SERIAL_CONFIG	EQU	RAM_BASE+$00E
SERIAL_WORKSPACE	EQU	RAM_BASE+$980	* Workspace for string processing
SERIAL_INBUF	EQU	RAM_BASE+$1000	* Serial input buffer
SERIAL_OUTBUF	EQU	SERIAL_INBUF+SERIAL_BUFLEN	* Serial output buffer
SERIAL_INEND	EQU	SERIAL_OUTBUF-2
SERIAL_OUTEND	EQU	SERIAL_INBUF+SERIAL_BUFLEN - 2

RTC_SC_SECS	EQU	RAM_BASE+$010	* Soft copy of Real Time Clock
RTC_SC_SECS_ALRM	EQU	RAM_BASE+$011
RTC_SC_MINS	EQU	RAM_BASE+$012
RTC_SC_MINS_ALRM	EQU	RAM_BASE+$013
RTC_SC_HRS		EQU	RAM_BASE+$014
RTC_SC_HRS_ALRM	EQU	RAM_BASE+$015
RTC_SC_DAY		EQU	RAM_BASE+$016
RTC_SC_DATE	EQU	RAM_BASE+$017
RTC_SC_MONTH	EQU	RAM_BASE+$018
RTC_SC_YEAR	EQU	RAM_BASE+$019

TF_BLK		EQU	RAM_BASE+$1200	* Block to hold timeframes
TF_BLK_END	EQU	RAM_BASE+$2000	* allow plenty of space
TF_START	EQU	RAM_BASE+$020	* Pointer to first
TF_END		EQU	RAM_BASE+$022	* Pointer to last

VIA_CA1_FLAG	EQU	RAM_BASE+$24	* Flag to show status of CA1
VIA_CB1_FLAG	EQU	RAM_BASE+$25	* Flag to show status of CB1
RTC_INT_EN	EQU	RAM_BASE+$26	* Flag to control RTC interrupts

*TF_DATE	EQU	$0		* Offset into timeframe
*TF_TIME	EQU	$4		* These would be the definitions if
*TF_DEVICE	EQU	$8		* dates were implemented
*TF_ACTION	EQU	$A
*TF_LEN		EQU	$E		* Length of timeframe block

TF_TIME		EQU	$0		* Offset into timeframe
TF_DEVICE	EQU	$4
TF_ACTION	EQU	$6
TF_LEN		EQU	$A		* Length of timeframe block

* Miscellaneous definitions

LF	EQU	10			* ASCII control codes
FF	EQU	12
CR	EQU	13
