	* 68000 interrupt service routines for
	* Theo Markettos' Home Automation System
	* 
	* This code is copyright Theo Markettos 1997,
	* and may be freely distributed, with the proviso
	* that it is used for educational and non-profit
	* making purposes only
	* 
	* If you have any problems using this code, you
	* should contact the author at
	* theomarkettos@letterbox.com

	ORG	$70			* PIA's interrupt vector
	DC.L	pia_int

	ORG	$74			* VIA's interrupt vector
	DC.L	via_int

	ORG	$7C			* NMI handler
	DC.L	memdump

	ORG	interrupt_code		* Centrally allocated code space

int_return				* general purpose return
	MOVEM.L	(A7)+,ALLREGS		* restore regs
	RTE				* return

pia_int					* Called on PIA interrupt
	MOVEM.L	ALLREGS,-(A7)		* Preserve all registers on stack
	MOVE.B	PIA_CRA,D0		* Read PIA status register A

	BTST	#7,D0			* Is IRQ A set
	BNE	rtc_int                 * its an RTC interrupt
	BTST	#6,D0			* Is IRQ B set
	NOP				* its something else's interrupt
	MOVE.B	PIA_CRB,D0		* Read PIA status register A
	BTST	#7,D0			* Is IRQ A set
	NOP		                * its something else's interrupt
	BTST	#6,D0			* Is IRQ B set
	NOP				* its something else's interrupt
	MOVEM.L	(A7)+,ALLREGS
	RTE

rtc_int
	*MOVE.B	D0,DOUT_H
	MOVE.L	#Hardware_RTC_Read,D0	* read the RTC to clear interrupt
	MOVE.B	#RTC_STATUS_C,D1	* status register C
	TRAP	#15			* do it
	BTST	#5,D0			* is it alarm interrupt?
	BNE	rtc_int_alarm
	BTST	#4,D0			* is it update interrupt?
	BNE	rtc_int_softcopy
	MOVEM.L	(A7)+,ALLREGS
	RTE

rtc_int_softcopy			* Copy the RTC values into RAM
	MOVE.L	#Hardware_RTC_Read,D0	* read the RTC seconds
	MOVE.B	#RTC_SECS,D1		* second register
	TRAP	#15			* do it
	MOVE.B	D0,RTC_SC_SECS		* store in memory
	MOVE.L	#Hardware_RTC_Read,D0	* do the same for all the other
	MOVE.B	#RTC_SECS_ALRM,D1	* timing registers
	TRAP	#15
	MOVE.B	D0,RTC_SC_SECS_ALRM
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_MINS,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_MINS
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_MINS_ALRM,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_MINS_ALRM
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_HRS,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_HRS
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_HRS_ALRM,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_HRS_ALRM
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_DAY,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_DAY
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_DATE,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_DATE
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_MONTH,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_MONTH
	MOVE.L	#Hardware_RTC_Read,D0
	MOVE.B	#RTC_YEAR,D1
	TRAP	#15
	MOVE.B	D0,RTC_SC_YEAR
	MOVEM.L	(A7)+,ALLREGS
	RTE

rtc_int_alarm
	MOVE.W	#Hardware_RTC_Read,D0	* read status register
	MOVEQ	#RTC_STATUS_B,D1
	TRAP	#15

	BCLR	#5,D0			* clear alarm bit - disable

	MOVE.L	D0,D2			* alarm interrupts
	MOVE.W	#Hardware_RTC_Write,D0	* write status register
	MOVEQ	#RTC_STATUS_B,D1
	TRAP	#15

	MOVE.W	#Hardware_RTC_Read,D0	* read device number
	MOVEQ	#RTC_DEVICE_ALRM,D1
	TRAP	#15

	MOVE.L	D0,D7
	MOVE.W	#Hardware_RTC_Read,D0	* read setting
	MOVEQ	#RTC_DATA_VALUE,D1
	TRAP	#15
					* now D7 = device to write
					* and D0 = value to write to it
	MOVE.L	D0,D1			* put values in correct registers
	MOVE.L	D7,D0

	BSR	send_PIC_command	* send to PIC

	MOVEM.L	(A7)+,ALLREGS
	RTE

via_int
	MOVEM.L	ALLREGS,-(A7)		* Preserve all registers on stack
	MOVE.B	VIA_IFR,D0		* Read VIA interrupt flags
	BTST	#4,D0			* Has CB1 interrupt occurred?
	BNE	via_cb1_int		* Handle it
	BTST	#2,D0			* Has CA1 interrupt occurred?
	BNE	via_ca1_int		* Handle it
	MOVEM.L	(A7)+,ALLREGS		* If neither, return
	RTE

via_ca1_int
	MOVE.B  VIA_DRA,D0              * Read register and clear interrupt
	MOVEM.L	(A7)+,ALLREGS		* then return
	RTE

via_cb1_int
	MOVE.B  VIA_DRB,D0              * Read register and clear interrupt
					* from 6522
	MOVE.B	VIA_PCR,D0		* Read current contents of PCR
	MOVE.B	D0,D1			* Preserve it
	BSET	#3,D0
	BSET	#2,D0                   * Take CA2 low
	BCLR	#1,D0
	MOVE.B	D0,VIA_PCR              * do it - clears PIC interrupt

        MOVEQ   #$7F,D7
via_cb1_int_delay
	DBF     D7,via_cb1_int_delay    * short delay loop

	MOVE.B  D1,VIA_PCR              * reset to original status

	MOVEM.L	(A7)+,ALLREGS		* then return
	RTE


memdump
	MOVEM.L ALLREGS,-(A7)		* Preserve registers on stack
	LEA	$0,A0			* address to start at
memdump_loop1
	MOVE.B	(A0)+,D0		* get byte from memory
	MOVE.B	D0,ACIA_TR		* transmit byte
memdump_get_tdre1
	MOVE.B	ACIA_SR,D1		* get ACIA status
	MOVE.B	D1,DOUT_H		* DEBUG CODE
	BTST	#1,D1			* is Tx data reg empty
	BEQ	memdump_get_tdre1	* if not wait until is
	CMPA.L	#$4000,A0		* is end of ROM
	BNE	memdump_loop1		* if not loop round
	LEA	$40000,A0		* address to start at
memdump_loop2
	MOVE.B	(A0)+,D0		* get byte from memory
	MOVE.B	D0,ACIA_TR		* transmit byte
memdump_get_tdre2
	MOVE.B	ACIA_SR,D1		* get ACIA status
	MOVE.B	D1,DOUT_H		* DEBUG CODE
	BTST	#1,D1			* is Tx data reg empty
	BEQ	memdump_get_tdre2	* if not wait until is
	CMPA.L	#$44000,A0		* is end of ROM
	BNE	memdump_loop2		* if not loop round
	MOVEM.L	(A7)+,ALLREGS		* Restore stacked registers
	RTE				* and return
